library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.numeric_std.all;
library work;
    use work.system_params.all;
    use work.matrix_types.all;
    use work.matrix_mult_functions.all;
    use work.memory_map.all;

entity null_syndrome is
    port(
        -- control signals
        clk_i:      in  std_logic;
        rst_n_i:    in  std_logic;
        start_i:    in  std_logic;
        -- to memory
        a_i:        in  pos;
        a_addr_o:   out addr;
        b_i:        in  pos;
        b_addr_o:   out addr;
        z_o:        out pos;
        z_addr_o:   out addr;
        wr_o:       out std_logic;
        -- report back when done
        null_syn_o: out std_logic;
        done_o:     out std_logic
    );
end entity null_syndrome;

architecture rtl of null_syndrome is

    type state_t is (IDLE, BUSY, DONE);
    signal  state:      state_t;
    signal  next_state: state_t;

    signal  bad_syn_c:  std_logic;
    signal  bad_syn_r:  std_logic;

    signal  i_c:    natural;
    signal  i_r:    natural;

begin

    a_addr_o    <=  S_BASE_ADDR + i_r;
    b_addr_o    <=  S_BASE_ADDR + i_r + 1;
    z_o         <=  to_unsigned(0, position_bits);
    z_addr_o    <=  S_BASE_ADDR;
    wr_o        <=  '0';
    null_syn_o  <=  not bad_syn_r;

    seq: process(clk_i, rst_n_i)
    begin
        if rst_n_i = '0' then
            state       <=  IDLE;
            bad_syn_r   <=  '0';
            i_r         <=  0;
        elsif rising_edge(clk_i) then
            state       <=  next_state;
            bad_syn_r   <=  bad_syn_c;
            i_r         <=  i_c;
        end if;
    end process seq;
    comb: process(state, start_i, a_i, b_i, i_r, bad_syn_c, bad_syn_r)
    begin
        next_state  <=  state;
        bad_syn_c   <=  bad_syn_r;
        i_c         <=  i_r;
        done_o      <=  '0';
        case state is
            when IDLE =>
                bad_syn_c   <=  '0';
                i_c         <=  0;
                if start_i = '1' then
                    next_state  <=  BUSY;
                end if;
            when BUSY =>
                i_c     <=  i_r + 2;
                if a_i /= 0 then
                    bad_syn_c   <=  '1';
                end if;
                if b_i /= 0 and i_r /= P-1 then
                    bad_syn_c   <=  '1';
                end if;
                if bad_syn_c = '1' or i_r = P-1 then
                    next_state  <=  DONE;
                end if;
            when DONE =>
                done_o      <=  '1';
                if start_i = '0' then
                    next_state  <=  IDLE;
                end if;
        end case;
    end process comb;

end architecture rtl;

