library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.numeric_std.all;
library work;
    use work.system_params.all;
    use work.matrix_types.all;
    use work.matrix_mult_functions.all;
    use work.memory_map.all;

entity clear_temp is
    port(
        -- control signals
        clk_i:      in  std_logic;
        rst_n_i:    in  std_logic;
        start_i:    in  std_logic;
        -- to memory
        a_i:        in  pos;
        a_addr_o:   out addr;
        b_i:        in  pos;
        b_addr_o:   out addr;
        z_o:        out pos;
        z_addr_o:   out addr;
        wr_o:       out std_logic;
        -- report back when done
        done_o:     out std_logic
    );
end entity clear_temp;

architecture rtl of clear_temp is

    type state_t is (
        IDLE, CLEAR_MUL_RES, CLEAR_SUM_TMP, CLEAR_SIGMA,
        CLEAR_R, DONE
    );
    signal  state:      state_t;
    signal  next_state: state_t;

    signal  i_c:        integer;
    signal  i_r:        integer;

begin

    a_addr_o    <=  HTR_BASE_ADDR;
    b_addr_o    <=  HTR_BASE_ADDR;
    z_o         <=  to_unsigned(0, position_bits);

    seq: process(clk_i, rst_n_i)
    begin
        if rst_n_i = '0' then
            state   <=  IDLE;
            i_r     <=  0;
        elsif rising_edge(clk_i) then
            state   <=  next_state;
            i_r     <=  i_c;
        end if;
    end process seq;
    comb: process(state, start_i, i_r)
    begin
        i_c     <=  i_r;
        done_o  <=  '0';
        case state is
            when IDLE =>
                i_c         <=  0;
                z_addr_o    <=  HTR_BASE_ADDR;
                wr_o        <=  '0';
                if start_i = '1' then
                    next_state  <=  CLEAR_MUL_RES;
                end if;
            when CLEAR_MUL_RES =>
                i_c         <=  i_r + 1;
                z_addr_o    <=  MUL_RES_BASE_ADDR + i_r;
                wr_o        <=  '1';
                if i_r = DV*max(M) - 1 then
                    i_c         <=  0;
                    next_state  <=  CLEAR_SUM_TMP;
                end if;
            when CLEAR_SUM_TMP =>
                i_c         <=  i_r + 1;
                z_addr_o    <=  SUM_TMP_BASE_ADDR + i_r;
                wr_o        <=  '1';
                if i_r = DV*sum(M) - 1 then
                    i_c         <=  0;
                    next_state  <=  CLEAR_SIGMA;
                end if;
            when CLEAR_SIGMA =>
                i_c         <=  i_r + 1;
                z_addr_o    <=  SIGMA_BASE_ADDR + i_r;
                wr_o        <=  '1';
                if i_r = N0*P - 1 then
                    i_c         <=  0;
                    next_state  <=  CLEAR_R;
                end if;
            when CLEAR_R =>
                i_c         <=  i_r + 1;
                z_addr_o    <=  MUL_RES_BASE_ADDR + i_r;
                wr_o        <=  '1';
                if i_r = N0*P - 1 then
                    i_c         <=  0;
                    next_state  <=  DONE;
                end if;
            when DONE =>
                i_c         <=  0;
                z_addr_o    <=  HTR_BASE_ADDR;
                wr_o        <=  '0';
                done_o      <=  '1';
                if start_i = '0' then
                    next_state  <=  IDLE;
                end if;
        end case;
    end process comb;

end architecture rtl;

