library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.numeric_std.all;
library work;
    use work.system_params.all;
    use work.matrix_types.all;
    use work.matrix_mult_functions.all;
    use work.memory_map.all;

entity comp_message is
    port(
        -- control signals
        clk_i:      in  std_logic;
        rst_n_i:    in  std_logic;
        start_i:    in  std_logic;
        -- to memory
        a_i:        in  pos;
        a_addr_o:   out addr;
        b_i:        in  pos;
        b_addr_o:   out addr;
        z_o:        out pos;
        z_addr_o:   out addr;
        wr_o:       out std_logic;
        -- report back when done
        done_o:     out std_logic
    );
end entity comp_message;

architecture rtl of comp_message is

    type state_t is (IDLE, BUSY, DONE);
    signal  state:      state_t;
    signal  next_state: state_t;

    signal  i_c, i_r:   integer;

begin

    a_addr_o    <=  X_BASE_ADDR + i_r;
    b_addr_o    <=  E_BASE_ADDR + i_r;
    z_o         <=  a_i xor b_i;
    z_addr_o    <=  U_BASE_ADDR + i_r;

    seq: process(clk_i, rst_n_i)
    begin
        if rst_n_i = '0' then
            state   <=  IDLE;
        elsif rising_edge(clk_i) then
            state   <=  next_state;
        end if;
    end process seq;
    comb: process(state, start_i, i_r)
    begin
        next_state  <=  state;
        i_c         <=  0;
        wr_o        <=  '0';
        done_o      <=  '0';
        case (state) is
            when IDLE =>
                if start_i = '1' then
                    next_state  <=  BUSY;
                end if;
            when BUSY =>
                i_c     <=  i_r + 1;
                wr_o    <=  '1';
                if i_r = N0*(P-1) - 1 then
                    next_state  <=  DONE;
                end if; 
            when DONE =>
                done_o  <=  '1';
                if start_i = '0' then
                    next_state  <=  IDLE;
                end if;
        end case;
    end process comb;

end architecture rtl;

