library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.numeric_std.all;
library work;
    use work.system_params.all;
    use work.matrix_types.all;
    use work.matrix_mult_functions.all;
    use work.memory_map.all;

entity find_max is
    port(
        -- control signals
        clk_i:      in  std_logic;
        rst_n_i:    in  std_logic;
        start_i:    in  std_logic;
        -- to memory
        a_i:        in  pos;
        a_addr_o:   out addr;
        b_i:        in  pos;
        b_addr_o:   out addr;
        z_o:        out pos;
        z_addr_o:   out addr;
        wr_o:       out std_logic;
        -- report back when done
        max_idx_o:  out n_array(15 downto 0);
        done_o:     out std_logic
    );
end entity find_max;

architecture rtl of find_max is

    type state_t is (IDLE, BUSY, DONE);
    signal  state:      state_t;
    signal  next_state: state_t;

    signal  a_index_c, a_index_r:   natural;

    signal  max_val_c, max_val_r:           pos;
    signal  max_indexes_c, max_indexes_r:   n_array(15 downto 0);

    signal  i_c, i_r:   natural;

    constant    INVALID:    natural:=   N0*P;

begin

    a_addr_o    <=  R_BASE_ADDR + a_index_r;
    b_addr_o    <=  to_unsigned(0, address_bits);
    z_o         <=  to_unsigned(0, position_bits);
    z_addr_o    <=  to_unsigned(0, address_bits);
    wr_o        <=  '0';
    max_idx_o   <=  max_indexes_r;

    seq: process(clk_i, rst_n_i)
    begin
        if rst_n_i = '0' then
            state           <=  IDLE;
            a_index_r       <=  0;
            max_val_r       <=  to_unsigned(0, position_bits);
            max_indexes_r   <=  (others => invalid);
            i_r             <=  0;
        elsif rising_edge(clk_i) then
            state           <=  next_state;
            a_index_r       <=  a_index_c;
            max_val_r       <=  max_val_c;
            max_indexes_r   <=  max_indexes_c;
            i_r             <=  i_c;
        end if;
    end process seq;
    comb: process(
        state, start_i,
        a_index_r, a_i,
        max_val_r, max_indexes_r, i_r)
    begin
        next_state      <=  state;
        done_o          <=  '0';
        a_index_c       <=  0;
        max_val_c       <=  max_val_r;
        max_indexes_c   <=  max_indexes_r;
        i_c             <=  i_r;
        case state is
            when IDLE =>
                max_val_c       <=  to_unsigned(0, position_bits);
                max_indexes_c   <=  (others => INVALID);
                i_c             <=  0;
                if start_i = '1' then
                    next_state  <=  BUSY;
                end if;
            when BUSY =>
                a_index_c   <=  a_index_r + 1;
                if a_i = max_val_r then
                    max_indexes_c(i_r)  <=  a_index_r;
                    -- if we have too many equal maxes we'll flip just some
                    if i_r /= 15 then
                        i_c                 <=  i_r + 1;
                    end if;
                elsif a_i > max_val_r then
                    max_val_c       <=  a_i;
                    max_indexes_c   <=  (0 => a_index_r, others => INVALID);
                    i_c             <=  1;
                end if;
                if a_index_r = N0*P-1 then
                    a_index_c   <=  0;
                    next_state  <=  DONE;
                end if;
            when DONE =>
                done_o  <=  '1';
                if start_i = '0' then
                    next_state  <=  IDLE;
                end if;
        end case;
    end process comb;

end architecture rtl;

