import random
try:
    import numpy
except:
    print("Numpy was not found on this machine, no dense representation is allowed.")
    print("Please do not set the \"--run_dense_comparison\" option.")

def generate_Htr(n0, p, dv):
    # Initialize H to an empty vector of correct size
    H = [None]*(n0*dv)
    index = 0
    for block in range(n0):
        set_positions = sorted(random.sample(range(p), dv))
        for pos in set_positions:
            H[index] = pos
            index += 1
    return H

def generate_Qtr(n0, p, m):
    # Initialize Q to an empty vector of correct size
    Q = [None]*(n0*sum(m))
    # Get full weight matrix
    m_copy = list(m)
    m_copy.reverse()
    mtr_exp = []
    for i in range(n0):
        mtr_exp.append(m_copy[-i - 1:] + m_copy[:-i - 1])
    index = 0
    for i in range(n0):
        for j in range(n0):
            set_positions = sorted(random.sample(range(p), mtr_exp[i][j]))
            for pos in set_positions:
                Q[index] = pos
                index += 1
    return Q

def Htr_sparse_to_dense(Htr, n0, p, dv):
    # Initialize Htr_dense to an empty matrix of correct size
    Htr_dense = numpy.zeros((n0*p, p), dtype=int)
    # Populate Htr_dense
    for block in range(n0):
        for pos in Htr[block*dv:(block+1)*dv]:
            for Htr_dense_row in range(block*p, (block+1)*p):
                Htr_dense[Htr_dense_row][(pos+Htr_dense_row) % p] = 1
    return Htr_dense

def Qtr_sparse_to_dense(Qtr, n0, p, m):
    # Initialize Qtr_dense to an empty matrix of correct size
    Qtr_dense = numpy.zeros((n0 * p, n0 * p), dtype=int)
    # Get weights for each block
    m_copy = list(m)
    m_copy.reverse()
    mtr_exp = []
    for i in range(n0):
        mtr_exp.append(m_copy[-i - 1:] + m_copy[:-i - 1])
    # Populate Qtr_dense
    for block_i in range(n0):
        for block_j in range(n0):
            start = block_i*sum(m) + sum(mtr_exp[block_i][:block_j])
            end = block_i*sum(m) + sum(mtr_exp[block_i][:block_j+1])
            for pos in Qtr[start:end]:
                for Qtr_dense_row in range(block_i*p, (block_i+1)*p):
                    Qtr_dense[Qtr_dense_row][(pos+Qtr_dense_row) % p + block_j*p] = 1
    return Qtr_dense

def Ltr_sparse_to_dense(Ltr, n0, p, dv, m):
    # Initialize Ltr_dense to an empty matrix of correct size
    Ltr_dense = numpy.zeros((n0 * p, p), dtype=int)
    # Populate Ltr_dense
    for block in range(n0):
        for pos in Ltr[block * sum(m)*dv:(block + 1) * sum(m)*dv]:
            # ignore invalid positions
            if pos == p:
                continue
            for Ltr_dense_row in range(block * p, (block + 1) * p):
                Ltr_dense[Ltr_dense_row][(pos + Ltr_dense_row) % p] = 1
    return Ltr_dense