library IEEE;
    use IEEE.std_logic_1164.all;
    use IEEE.numeric_std.all;
library work;
    use work.system_params.all;
    use work.matrix_types.all;
    use work.matrix_mult_functions.all;
    use work.memory_map.all;
    use work.mem_io.all;

entity key_reconstruction_tb is
end entity key_reconstruction_tb;

architecture test of key_reconstruction_tb is

    component key_reconstruction is
        port (
            -- control signals
            clk_i:          in  std_logic;
            rst_n_i:        in  std_logic;
            start_i:        in  std_logic;
            -- data, addresses and controls to memory
            a_i:            in  pos;
            a_addr_o:       out addr;
            b_i:            in  pos;
            b_addr_o:       out addr;
            z_o:            out pos;
            z_addr_o:       out addr;
            wr_o:           out std_logic;
            -- report back once done
            key_rec_done_o: out std_logic
        );
    end component key_reconstruction;

    component RAM is
        generic(
            ram_size:   natural:=   1024
        );
        port(
            clk_i:      in  std_logic;
            rst_n_i:    in  std_logic;
            wr_i:       in  std_logic;
            addr1_i:    in  addr;
            data1_o:    out pos;
            addr2_i:    in  addr;
            data2_o:    out pos;
            addr3_i:    in  addr;
            data3_i:    in  pos
        );
    end component RAM;

    component clk_gen is
        generic(
            period: time
        );
        port(
            rst_n_i:    in  std_logic;
            clk_o:      out std_logic
        );
    end component clk_gen;

    signal  clk:            std_logic;
    signal  rst_n:          std_logic:= '0';
    signal  start:          std_logic;
    signal  a:              pos;
    signal  a_addr:         addr;
    signal  b:              pos;
    signal  b_addr:         addr;
    signal  z:              pos;
    signal  z_addr:         addr;
    signal  wr:             std_logic;
    signal  key_rec_done:   std_logic;

begin

    KR: key_reconstruction
        port map(
            clk_i           =>  clk,
            rst_n_i         =>  rst_n,
            start_i         =>  start,
            a_i             =>  a,
            a_addr_o        =>  a_addr,
            b_i             =>  b,
            b_addr_o        =>  b_addr,
            z_o             =>  z,
            z_addr_o        =>  z_addr,
            wr_o            =>  wr,
            key_rec_done_o  =>  key_rec_done
        );

    MEM: RAM
        generic map(
            ram_size        =>  2048
        )
        port map(
            clk_i           =>  clk,
            rst_n_i         =>  rst_n,
            wr_i            =>  wr,
            addr1_i         =>  a_addr,
            data1_o         =>  a,
            addr2_i         =>  b_addr,
            data2_o         =>  b,
            addr3_i         =>  z_addr,
            data3_i         =>  z
        );

    CG: clk_gen
        generic map(
            period          =>  10 ns
        )
        port map(
            rst_n_i         =>  rst_n,
            clk_o           =>  clk
        );

    rst_n   <=  '1' after 10 ns;

    stim: process
    begin
        load_in_memory(
            to_integer(HTR_BASE_ADDR),
            "matrices/Htr.txt",
            clk,
            z_addr,
            z,
            wr
        );
        load_in_memory(
            to_integer(QTR_BASE_ADDR),
            "matrices/Qtr.txt",
            clk,
            z_addr,
            z,
            wr
        );
        a_addr  <=  (others => 'Z');
        start   <=  '1';
        wait until key_rec_done = '1';
        start   <=  '0';
        memory_dump(
            to_integer(LTR_BASE_ADDR),
            to_integer(LTR_BASE_ADDR) + 237,
            "matrices/Ltr_sim.txt",
            clk,
            a_addr,
            a,
            wr
        );
        wait;

    end process stim;

end architecture test;

