library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.std_logic_textio.all;
use std.textio.all;
use work.system_params.all;
use work.matrix_types.all;
use work.memory_map.all;
use work.matrix_mult_functions.all;

package mem_io is
    procedure load_in_memory(
        -- arguments
        constant    base_addr:  in  integer;
        constant    file_name:  in  string;
        -- signals
        signal      clk:        in  std_logic;
        signal      addr_out:   out addr;
        signal      data_out:   out pos;
        signal      data_wr:    out std_logic
    );
    procedure memory_dump(
        -- arguments
        constant    start_addr: in  integer;
        constant    end_addr:   in  integer;
        constant    file_name:  in  string;
        -- signals
        signal      clk:        in  std_logic;
        signal      addr_out:   out addr;
        signal      data_in:    in  pos;
        signal      data_wr:    out std_logic
    );
end package;

package body mem_io is

    -- TODO: untested
    procedure load_in_memory(
        -- arguments
        constant    base_addr:  in  integer;
        constant    file_name:  in  string;
        -- signals
        signal      clk:        in  std_logic;
        signal      addr_out:   out addr;
        signal      data_out:   out pos;
        signal      data_wr:    out std_logic
    ) is
        file        fd:         text;
        variable    data_line:  line;
        variable    data_var:   std_logic_vector(position_bits-1 downto 0);
        variable    index:      addr;
    begin
        file_open(fd, file_name, read_mode);
        index   :=  to_unsigned(base_addr, address_bits);
        report "starting memory load from file " & file_name &
            " at address " & integer'image(to_integer(index));
        data_wr <=  force in '1';
        data_wr <=  force out '1';
        while not endfile(fd) loop
            readline(fd, data_line);
            read(data_line, data_var);
            addr_out    <=  force in index;
            addr_out    <=  force out index;
            data_out    <=  force in unsigned(data_var);
            data_out    <=  force out unsigned(data_var);
            wait until rising_edge(clk);
            addr_out    <=  release;
            data_out    <=  (others => 'Z');
            data_out    <=  release;
            addr_out    <=  (others => 'Z');
            index       :=  index + 1;
        end loop;
        data_wr <=  release;
        data_wr <=  'Z';
        report "finished loading at address" & integer'image(to_integer(index));
        file_close(fd);
    end procedure load_in_memory;

    -- TODO: untested
    procedure memory_dump(
        -- arguments
        constant    start_addr: in  integer;
        constant    end_addr:   in  integer;
        constant    file_name:  in  string;
        -- signals
        signal      clk:        in  std_logic;
        signal      addr_out:   out addr;
        signal      data_in:    in  pos;
        signal      data_wr:    out std_logic
    ) is
        file        fd:         text;
        variable    data_line:  line;
        variable    index:      addr;
    begin
        file_open(fd, file_name, write_mode);
        index   :=  to_unsigned(start_addr, address_bits);
        report "starting memory dump to file " & file_name &
            " at address " & integer'image(to_integer(index));
        data_wr <=  force '0';
        while index <= end_addr loop
            addr_out    <=  force index;
            wait until rising_edge(clk);
            write(data_line, data_in);
            writeline(fd, data_line);
            addr_out    <=  release;
            addr_out    <=  (others => 'Z');
            index       :=  index + 1;
        end loop;
        data_wr <=  release;
        data_wr <=  'Z';
        report "finished dumping at address" & integer'image(to_integer(index));
        file_close(fd);
    end procedure memory_dump;

end package body;
