library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.system_params.all;
use work.matrix_types.all;

entity RAM is
    generic(
        ram_size:   natural:=   1024
    );
    port(
        clk_i:      in  std_logic;
        rst_n_i:    in  std_logic;
        wr_i:       in  std_logic;
        addr1_i:    in  addr;
        data1_o:    out pos;
        addr2_i:    in  addr;
        data2_o:    out pos;
        addr3_i:    in  addr;
        data3_i:    in  pos
    );
end entity RAM;

architecture rtl of RAM is
    type memory_t is array(natural range <>) of pos;
    signal  memory: memory_t(0 to ram_size-1);
begin

    check_addr: process(rst_n_i, addr1_i, addr2_i, addr3_i)
    begin
        if rst_n_i = '1' then -- if we are not reset check values
            assert addr1_i < ram_size report "address " & integer'image(to_integer(addr1_i)) & " out of bounds";
            assert addr2_i < ram_size report "address " & integer'image(to_integer(addr2_i)) & " out of bounds";
            assert addr3_i < ram_size report "address " & integer'image(to_integer(addr3_i)) & " out of bounds";
        end if;
    end process check_addr;

    data1_o     <=  memory(to_integer(addr1_i));
    data2_o     <=  memory(to_integer(addr2_i));

    wr_i_seq: process(clk_i, rst_n_i)
    begin
        if rst_n_i = '0' then
            memory  <=  (others => (others => '0'));
        elsif rising_edge(clk_i) then
            if wr_i = '1' then
                memory(to_integer(addr3_i)) <=  data3_i;
            else
                memory(to_integer(addr3_i)) <=  memory(to_integer(addr3_i));
            end if;
        end if;
    end process wr_i_seq;

end architecture rtl;
